// testies.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>
#include <stdio.h>
#include "RtAudio.h"
#include <iostream>
#include <string>
#include <vector>
#include <assert.h>
#include "asiosys.h"
#include <conio.h>
using namespace std;

// define sample
#define SAMPLE double
#define MY_PIE 3.14159265358979
#define MY_SRATE 44100

SAMPLE * g_buffy = NULL;
SAMPLE * g_hann = NULL;
SAMPLE * g_hamming = NULL;

int numChannels = 0;
int numDevice = 0;

// audio call back
int callme( void * out, void * in, unsigned int numFrames,
            double streamTime, RtAudioStreamStatus status, void * userData )
{
    // warn in case of over/underflow
    if ( status ) std::cout << "Stream over/underflow detected." << std::endl;
    
    // fprintf( stderr, "." );
    SAMPLE * buffy = (SAMPLE *)out;
	SAMPLE * input = (SAMPLE *)in;
    
    // loop over
    for( unsigned int i = 0; i < numFrames; i++ )
    {
        // copy the input to output
//        buffy[2*i] = buffy[2*i+1] = input[i*2];
		buffy[i] = input[i];		
//		cerr << i << " : " << input[i] << endl;
	}

    // copy | TODO: unhack this from hardcoding for stereo
    memcpy( g_buffy, in, sizeof(SAMPLE)*numFrames*numChannels );
    
    return 0;    
}



//-----------------------------------------------------------------------------
// function prototypes
//-----------------------------------------------------------------------------
void idleFunc( );
void displayFunc( );
void reshapeFunc( GLsizei width, GLsizei height );
void keyboardFunc( unsigned char, int, int );
void mouseFunc( int button, int state, int x, int y );

void initialize( );
void cleanUp( );




//-----------------------------------------------------------------------------
// global variables and #defines
//-----------------------------------------------------------------------------
#define __PI    3.1415926

const float g_deg2rad = __PI/180;
// width and height of the window
GLsizei g_width = 800;
GLsizei g_height = 600;

// light 0 position
GLfloat g_light0_pos[4] = { 2.0f, 8.2f, 4.0f, 1.0f };

// light 1 parameters
GLfloat g_light1_ambient[] = { .2f, .2f, .2f, 1.0f };
GLfloat g_light1_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat g_light1_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat g_light1_pos[4] = { -2.0f, 0.0f, -4.0f, 1.0f };

// RtAudio object
RtAudio g_audio;
// buffer frames
unsigned int g_bufferFrames;


//-----------------------------------------------------------------------------
// name: make_circle()
// desc: draw a damn circle
//-----------------------------------------------------------------------------
void make_circle(float radius, int chan)
{
	for( unsigned int i = 0; i < g_bufferFrames; i++ )
    {
	glBegin( GL_LINE_STRIP );
	float degINrad = i*g_deg2rad;    
	glVertex2f( cos(degINrad)*(g_buffy[i*numChannels+chan]+radius), (sin(degINrad)*(g_buffy[i*numChannels+chan]+radius) )); 
	//cerr << i*numChannels+chan << endl;
	}
	glEnd();
}

//-----------------------------------------------------------------------------
// name: hanning()
// desc: make window
//-----------------------------------------------------------------------------
void hanning( SAMPLE * window, unsigned long length )
{
    unsigned long i;
    double pi, phase = 0, delta;
    
    pi = 4.*atan(1.0);
    delta = 2 * pi / (double) length;
    
    for( i = 0; i < length; i++ )
    {
        window[i] = (0.5 * (1.0 - cos(phase)));
        phase += delta;
    }
}




//-----------------------------------------------------------------------------
// name: hamming()
// desc: make window
//-----------------------------------------------------------------------------
void hamming( SAMPLE * window, unsigned long length )
{
    unsigned long i;
    double pi, phase = 0, delta;
    
    pi = 4.*atan(1.0);
    delta = 2 * pi / (double) length;
    
    for( i = 0; i < length; i++ )
    {
        window[i] = (0.54 - .46*cos(phase));
        phase += delta;
    }
}


//-----------------------------------------------------------------------------
// Name: main( )
// Desc: starting point
//-----------------------------------------------------------------------------
int main( int argc, char ** argv )
{
	
    // initialize GLUT
    glutInit( &argc, argv );
    // double buffer, use rgb color, enable depth buffer
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    // initialize the window size
    glutInitWindowSize( g_width, g_height );
    // set the window postion
    glutInitWindowPosition( 100, 100 );
    // create the window
	glutCreateWindow( "3 Testies: I heart openGL" );

    // set the idle function - called when idle
    glutIdleFunc( idleFunc );
    // set the display function - called when redrawing
    glutDisplayFunc( displayFunc );
    // set the reshape function - called when client area changes
    glutReshapeFunc( reshapeFunc );
    // set the keyboard function - called on keyboard events
    glutKeyboardFunc( keyboardFunc );
    // set the mouse function - called on mouse stuff
    glutMouseFunc( mouseFunc );

    // do our own initialization
    initialize();

    // let GLUT handle the current thread from here
    glutMainLoop();

    return 0;
}




//-----------------------------------------------------------------------------
// Name: initialize( )
// Desc: sets initial OpenGL states
//       also initializes any application data
//-----------------------------------------------------------------------------
void initialize()
{
	RtAudio::DeviceInfo info;
	for (unsigned int i=0; i<g_audio.getDeviceCount(); i++) {
    info = g_audio.getDeviceInfo(i);
	
	cerr << endl;
	cerr << "Device # " << i+1 << endl;
    std::cout << "\nDevice Name = " << info.name << '\n';
    if ( info.probed == false )
      std::cout << "Probe Status = UNsuccessful\n";
    else {
      std::cout << "Probe Status = Successful\n";
      std::cout << "Output Channels = " << info.outputChannels << '\n';
      std::cout << "Input Channels = " << info.inputChannels << '\n';
      std::cout << "Duplex Channels = " << info.duplexChannels << '\n';
      
      }
	}
	cerr << endl;
	cerr << "which audio device do you want to use?" << endl;
	cerr << "you have " << g_audio.getDeviceCount() << " audio devices" << endl;
	cin >> numDevice;
	cerr << endl;
	if( numDevice > g_audio.getDeviceCount())
	{
		cerr << endl;
		cerr << "Device #"<<numDevice << "?" << endl;
		cerr << "Hows about you go learn to recognize numbers and simple shapes" << endl;
		cerr << "and then give it another go eh?" << endl;
		getch();
		exit(1);
	}
	numDevice -= 1; 
	info = g_audio.getDeviceInfo(numDevice);
	cerr << "How many channels do you want to use?" << endl;
	cerr << "you have " << info.inputChannels << " audio channels" << endl;
	cin >> numChannels;

	if( (unsigned int)numChannels > info.inputChannels){
		cerr << endl;
		cerr << "nice try loser but you only have " << g_audio.getDeviceCount()<< " inputs" << endl;
		cerr << endl;
		cerr << "catch ya on the flip" << endl;
		cerr << endl;
		getch();
		cerr << "(and try not to screw it up next time)" << endl;
		exit(1);
	}
    // set the GL clear color - use when the color buffer is cleared
    glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
    // set the shading model to 'smooth'
    glShadeModel( GL_SMOOTH );
    // enable depth
    glEnable( GL_DEPTH_TEST );
    // set the front faces of polygons
    glFrontFace( GL_CCW );
    // set fill mode
    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
    
    // enable lighting
    glEnable( GL_LIGHTING );
    // enable lighting for front
    glLightModeli( GL_FRONT_AND_BACK, GL_TRUE );
    // material have diffuse and ambient lighting 
    glColorMaterial( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
    // enable color
    glEnable( GL_COLOR_MATERIAL );
    
    // enable light 0
    glEnable( GL_LIGHT0 );
    
    // setup and enable light 1
    glLightfv( GL_LIGHT1, GL_AMBIENT, g_light1_ambient );
    glLightfv( GL_LIGHT1, GL_DIFFUSE, g_light1_diffuse );
    glLightfv( GL_LIGHT1, GL_SPECULAR, g_light1_specular );
    // glEnable( GL_LIGHT1 );
    
    // set the position of the lights
    glLightfv( GL_LIGHT0, GL_POSITION, g_light0_pos );
    // glLightfv( GL_LIGHT1, GL_POSITION, g_light1_pos );
    
    // seed random number generator
    //srand( time(NULL) );
    
    // test for number of devices
    if( g_audio.getDeviceCount() < 1 )
    {
        cerr << "no audio device found!\n" << endl;
        exit( 1 );
    }
//    cerr << g_audio.getDeviceCount() << endl;
    // parameters
    unsigned int bufferFrames = 512;
    RtAudio::StreamParameters iParams, oParams;
    iParams.deviceId = numDevice;//g_audio.getDefaultInputDevice();
    iParams.nChannels = numChannels;
    oParams.deviceId = numDevice;//g_audio.getDefaultOutputDevice();
    oParams.nChannels = numChannels;

    
    // open the RtAudio
    try {
        g_audio.openStream( &oParams, &iParams, RTAUDIO_FLOAT64,
                         48000, &bufferFrames, &callme, NULL );
    } catch( RtError & err ) {
        err.printMessage();
        exit(1);
    }
    
    // allocate the buffer
    g_buffy = new SAMPLE[oParams.nChannels*bufferFrames];
    memset( g_buffy, 0, sizeof(SAMPLE)*oParams.nChannels*bufferFrames );
    assert( g_buffy != NULL );
    // remember the buffer frame
    g_bufferFrames = bufferFrames;
    // generate windows
    g_hann = new SAMPLE[bufferFrames]; // mono only
    hanning( g_hann, bufferFrames );
    g_hamming = new SAMPLE[bufferFrames]; // mono only
    hamming( g_hamming, bufferFrames );
    
    // start the stream
    try {
        // start
        g_audio.startStream();
    } catch( RtError & err ) {
        // do stuff
        err.printMessage();
    }
	
}




//-----------------------------------------------------------------------------
// Name: cleanUp( )
// Desc: cleans up dynamic memory for vArray
//-----------------------------------------------------------------------------
void cleanUp( )
{
    // TODO: !!!
}





//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void reshapeFunc( GLsizei w, GLsizei h )
{
    // save the new window size
    g_width = w; g_height = h;
    // map the view port to the client area
    glViewport( 0, 0, w, h );
    // set the matrix mode to project
    glMatrixMode( GL_PROJECTION );
    // load the identity matrix
    glLoadIdentity( );
    // create the viewing frustum
    gluPerspective( 45.0, (GLfloat) w / (GLfloat) h, 1.0, 300.0 );
    // set the matrix mode to modelview
    glMatrixMode( GL_MODELVIEW );
    // load the identity matrix
    glLoadIdentity( );
    // position the view point
    gluLookAt( 2.0f, 0.0f, 3.0f, 2.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
	
}




//-----------------------------------------------------------------------------
// name: rand2f()
// desc: generate a random float
//-----------------------------------------------------------------------------
GLfloat rand2f( float a, float b )
{
    return a + (b-a)*(rand() / (GLfloat)RAND_MAX);
}




//-----------------------------------------------------------------------------
// Name: keyboardFunc( )
// Desc: key event
//-----------------------------------------------------------------------------
void keyboardFunc( unsigned char key, int x, int y )
{
    switch( key )
    {
    case 'Q':
    case 'q':
        exit(1);
    break;
    }

    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: mouseFunc( )
// Desc: handles mouse stuff
//-----------------------------------------------------------------------------
void mouseFunc( int button, int state, int x, int y )
{
    if( button == GLUT_LEFT_BUTTON )
    {
        // when left mouse button is down, move left
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else if ( button == GLUT_RIGHT_BUTTON )
    {
        // when right mouse button down, move right
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else
    {
    }

    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: idleFunc( )
// Desc: callback from GLUT
//-----------------------------------------------------------------------------
void idleFunc( )
{
    // render the scene
    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: displayFunc( )
// Desc: callback function invoked to draw the client area
//-----------------------------------------------------------------------------
void displayFunc( )
{
    // static variable
    static GLfloat rot = 0.0f, drot = 0.0f, arot = 0.0f, mrot = 0.0f;
	
	//glRotatef(25,0,1,0);
    
	// clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
    // set line width
    glLineWidth( 5.0f );

	// save the current transformation
    glPushMatrix();
	glTranslatef(0.0f, 0.0f, (float) -numChannels*0.7f);
	for( int i = 0; i < numChannels; i++ )
	{
		glColor3f( 1.0f, 0.0f, 0.0f);
		glTranslatef(1.0f, 0.0f, 0.0f);
			make_circle(0.5f, i);
	}
/*
	glTranslatef(-1.0f, -0.5f, 0.0f);
    glColor3f( 0.4f, 1.0f, .2f);
	make_circle(1.0f, 0);

	glTranslatef(2.0f, 0.0f, 0.0f);	
	glColor3f(1.0f, .0f, .0f);
	make_circle(0.5f, 1);
    
	glTranslatef(0.0f, 2.0f, 0.0f);
	glColor3f(0.4f, 1.0f, 0.2f);
	make_circle(1.0f, 2);

	glTranslatef(-2.0f, 0.0f, 0.0f);
    glColor3f(1.0f, 0.0f, 0.0f);
	make_circle(0.5f, 3);

	//glTranslatef(-2.0f, 0.0f, 0.0f);
    glColor3f(1.0f, 0.0f, 0.0f);
	make_circle(0.25f, 4);
*/	
	glEnd();

    // restore state
    glPopMatrix();
    
    // flush!
    glFlush( );
    // swap the double buffer
    glutSwapBuffers( );
}
